#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "ptprobes.h"

static int nhits;
static const char *prog;
static struct ptp_process process;
static struct ptp_probe probe;
static pid_t pid;
static unsigned long addr;

static void usage()
{
	fprintf(stderr, "Usage: %s pid addr\n", prog);
	exit(1);
}

#define ckerr(err, func) if (err != 0) { fprintf(stderr, "%s returned %d\n", func, err); exit(2); } else fprintf(stderr, "%s succeeded\n", func);

#define trace_entry() fprintf(stderr, "Entering %s\n", __FUNCTION__)

void probe_handler(struct ptp_probe *p)
{
	nhits++;
}

void attach_handler(struct ptp_process *p)
{
	int err;
	probe.pid = p->pid;
	probe.addr = addr;
	probe.handler = probe_handler;
	err = ptp_register_probe(&probe);
	ckerr(err, "ptp_register_probe");
}

void exit_handler(struct ptp_process *p)
{
	fprintf(stderr, "Process %d exited.\n", p->pid);
	fprintf(stderr, "%d probe hits detected\n", nhits);
	exit(0);
}

main(int argc, char **argv)
{
	int err;
	char *endptr;

	prog = argv[0];
	if (argc != 3)
		usage();
	if ((pid = atoi(argv[1])) <= 1)
		usage();
	addr = strtoul(argv[2], &endptr, 0);
	if (endptr == argv[2] || *endptr != '\0')
		usage();

	err = ptp_init();
	ckerr(err, "ptp_init");

	process.pid = pid;
	process.attach_handler = attach_handler;
	process.exit_handler = exit_handler;
	err = ptp_attach_process(&process);
	ckerr(err, "ptp_attach_process");

	ptp_run();
	/*NOTREACHED*/
}
