/*
 * ptrace-based user-space probes
 * Copyright (c) International Business Machines Corp., 2006
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  Project Website:  http://sourceware.org/systemtap
 *
 */
#ifndef _PTPROBES_H
#define _PTPROBES_H

#include <sys/types.h>
#include <sys/queue.h>

typedef unsigned long ptp_addr_t;

// Group of probes at same address
struct ptp_aggr_probe;
struct ptp_aggr_probe_bucket;

struct ptp_probe {
	/*** Interface ***/
	pid_t pid;
	ptp_addr_t addr;
	void (*handler)(struct ptp_probe*);

	/*** Implementation ***/
	TAILQ_ENTRY(ptp_probe) list;
	struct ptp_aggr_probe *aggr;
};

enum ptp_process_state {
	ptpc_unattached,
	ptpc_attached,
	ptpc_probing,
	ptpc_exited
};

struct ptp_process {
	/*** Interface ***/
	pid_t pid;
	/*
	 * attach_handler is called when the child is stopped after attach.
	 * attach_handler should call ptp_register_probe() to set up
	 * probepoints.
	 */
	void (*attach_handler)(struct ptp_process*);
	void (*exit_handler)(struct ptp_process*);

	/*** Implementation ***/
	enum ptp_process_state state;
	LIST_ENTRY(ptp_process) list;
	struct ptp_aggr_probe_bucket *probe_hash;
	struct ptp_aggr_probe *active_probe;
};

extern int ptp_init(void);
extern int ptp_attach_process(struct ptp_process *p);
extern void ptp_run(void);
extern int ptp_register_probe(struct ptp_probe *p);
extern void ptp_unregister_probe(struct ptp_probe *p);

#endif /* _PTPROBES_H */
