/*
 * ptrace-based user-space probes
 * Copyright (c) International Business Machines Corp., 2006
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *  Project Website:  http://sourceware.org/systemtap
 *
 */

/* Architecture-independent implementation details */

#ifndef _PTPROBES_IMPL_H
#define _PTPROBES_IMPL_H

#include <sys/types.h>
#include <sys/reg.h>
#include <signal.h>
#include <sys/queue.h>
#include "ptprobes.h"
#include "arch/ptprobes_impl.h"

struct sigchld {
	int signo;	// always SIGCHLD
	int reason;	// e.g., CLD_TRAPPED.  See sigaction(2).
	int realsig;	// what somebody's trying to deliver to the child
	pid_t pid;	// child's pid
	long id;	// sequence # for debugging
	TAILQ_ENTRY(sigchld) list;
};

/*
 * Can have multiple probes at same address.  Every hashed probe
 * belongs to an aggregate.
 */
TAILQ_HEAD(ptp_probe_q, ptp_probe);

struct ptp_aggr_probe {
	pid_t pid;
	unsigned long addr;
	opcode_t opcode;
	struct ptp_probe_q probes;
	LIST_ENTRY(ptp_aggr_probe) list;
};

LIST_HEAD(ptp_aggr_probe_bucket, ptp_aggr_probe);

#endif /* _PTPROBES_IMPL_H */
